@@ -109,14 +109,18 @@ func (p *interp) getOutputStream(redirect Token, destValue value) (io.Writer, er
 
 	switch redirect {
 	case GREATER, APPEND:
+		// Write or append to file
 		if name == "-" {
 			// filename of "-" means write to stdout, eg: print "x" >"-"
 			return p.output, nil
 		}
-		// Write or append to file
 		if p.noFileWrites {
 			return nil, newError("can't write to file due to NoFileWrites")
 		}
+		if runtime.GOOS == "windows" && name == "/dev/stderr" {
+			// Special case /dev/stderr on Windows to allow portably printing to stderr.
+			return p.errorOutput, nil
+		}
 		p.flushOutputAndError() // ensure synchronization
 		flags := os.O_CREATE | os.O_WRONLY
 		if redirect == GREATER {
@@ -646,6 +650,7 @@ func (p *interp) ensureFields() {
 
 	switch {
 	case p.inputMode == CSVMode || p.inputMode == TSVMode:
+		// Normally fields have already been parsed by csvSplitter
 		if p.reparseCSV {
 			scanner := bufio.NewScanner(strings.NewReader(p.line))
 			scanner.Buffer(nil, maxRecordLength)
@@ -659,8 +664,6 @@ func (p *interp) ensureFields() {
 			if !scanner.Scan() {
 				p.fields = nil
 			}
-		} else {
-			// Normally fields have already been parsed by csvSplitter
 		}
 	case p.fieldSep == " ":
 		// FS space (default) means split fields on any whitespace
